#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)

import numpy as np
from . import dili


def get_nlabels(slab, labels, labels_meta=None, return_mode="num", return_first=False):
    """
    Get one or more labels, create a new one if necessary and return its numeric value.

    Look at the get_nlabel function for more details.

    :param slab:
    :param labels:
    :param labels_meta:
    :param return_mode: "num" or "str" or "both". Both means (numlabel, strlabel).
    :param return_first: Return just first found label
    :return:
    """

    if type(labels) not in (list, np.ndarray):
        labels = [labels]
        labels_meta = [labels_meta]
        return_first = True

    if labels_meta is None:
        labels_meta = [None] * len(labels)

    nlabels = []
    for label, label_meta in zip(labels, labels_meta):
        nlab = get_nlabel(slab, label, label_meta, return_mode=return_mode)
        nlabels.append(nlab)

    if return_first:
        nlabels = nlabels[0]
    return nlabels

def get_free_numeric_label(slab, minimum=1):
    i = minimum
    while i in slab.values():
        i += 1
    return i

def get_nlabel(slab, label, label_meta=None, return_mode="num", min_free_label=1):
    """
    Add label if it is necessery and return its numeric value.

    If "new" keyword is used and no other information is provided, the first free label is created.
    If "new" keyword is used and additional numeric info is provided, the number is used also as a key.
    :param return_mode: Set requested label return type. "int", "num", "numeric" or "str" or "both".
    "both" means (numlabel, strlabel).
    :param label: string, number or "new"
    :param label_meta: string, number or "new
    :param min_free_label: minimal value for label when new is created
    :return:
    """
    # todo add add_new and dont_add parameters to have fine control over adding new keys in slab
    numlabel = None
    strlabel = None
    if type(label) == str:
        if label_meta is None:
            if label not in slab.keys():
                free_numeric_label = get_free_numeric_label(slab, minimum=min_free_label)
                # free_numeric_label = np.max(list(slab.values())) + 1
                if label == "new":
                    label = str(free_numeric_label)
                slab[label] = free_numeric_label
                strlabel = label
                numlabel = slab[label]
            else:
                strlabel = label
                numlabel = slab[label]
        else:
            if label == "new":
                label = str(label_meta)
            update_slab(slab, label_meta, label)
            strlabel = label
            numlabel = label_meta
    else:
        # it is numeric
        if label_meta is None:
            if label not in list(slab.values()):
                update_slab(slab, label, str(label))
                strlabel = str(label)
            else:
                strlabel = dili.dict_find_key(slab, label)

            numlabel = label

        else:
            if label_meta == "new":
                label_meta = str(label)
            update_slab(slab, label, label_meta)
            strlabel = label_meta
            numlabel = label
            # return label

    if return_mode in ("num", "int", "numeric"):
        return numlabel
    elif return_mode == "str":
        return strlabel
    elif return_mode == "both":
        return numlabel, strlabel
    else:
        logger.error("Unknown return_mode: " + str(return_mode))


def update_slab(slab, numeric_label, string_label):
    """ Add label to segmentation label dictionary if it is not there yet.

    :param numeric_label:
    :param string_label:
    :return:
    """

    slab_tmp = {string_label: numeric_label}
    slab.update(slab_tmp)
    # slab = slab_tmp
    logger.debug('self.slab')
    logger.debug(str(slab))


def add_slab_label_carefully(slab, numeric_label, string_label):
    """ Add label to slab if it is not there yet.

    :param numeric_label:
    :param string_label:
    :return:
    """
    DeprecationWarning("Function will be removed in the future. Use get_nlabel instead.")
    get_nlabel(slab, numeric_label, string_label)


def add_missing_labels(segmentation, slab):
    labels = np.unique(segmentation)
    get_nlabels(slab, labels)

def minimize_slab(slab, segmentation, remove_doubled=True):
    """
    Check slab and kick out all not used or doubled values.
    :param slab:
    :param segmentation:
    :return:
    """
    un = np.unique(segmentation)
    unslab = np.unique(list(slab.values()))

    kick_labels = []
    keep_values = []
    for label in slab:
        val = slab[label]
        if val in un:
            if remove_doubled:
                if val in keep_values:
                    # it is doubled
                    kick_labels.append(val)
                else:
                    keep_values.append(label)
        else:
            kick_labels.append(label)

    for kicki in kick_labels:
        slab.pop(kicki)

    return slab




#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
import numpy as np
import scipy
import scipy.ndimage


def rotate(data3d, phi_deg, theta_deg=None, phi_axes=(1, 2), theta_axes=(0, 1), order=1, **kwargs):
    """
    Rotate 3D data by use angle and its axes or two angles.

    :param data3d: ndimage 3D
    :param phi_deg: deg
    :param phi_axes: deg
    :param theta_deg: deg
    :param theta_axes: deg
    :param order: optional, int. Default is 1. The order of the spline interpolation. See scipy for more details.
    :param kwargs: See scipy.ndimage.interpolation.rotate for more options
    :return:
    """

    data3d = scipy.ndimage.interpolation.rotate(data3d, phi_deg, phi_axes, order=order, **kwargs)
    if theta_deg is not None:
        data3d = scipy.ndimage.interpolation.rotate(data3d, theta_deg, theta_axes, order=order, **kwargs)
    return data3d
    # segmentation = scipy.ndimage.interpolation.rotate(segmentation, angle, axes)
    # seeds = scipy.ndimage.interpolation.rotate(seeds, angle, axes)


def random_rotate_paramteres():
    """
    Rotate data3d, segmentation and seeds with random rotation
    :return:
    """
    xi1 = np.random.rand()
    xi2 = np.random.rand()

    # theta = np.arccos(np.sqrt(1.0-xi1))
    theta = np.arccos(1.0 - (xi1 * 1))
    phi = xi2 * 2 * np.pi

    # xs = np.sin(theta) * np.cos(phi)
    # ys = np.sin(theta) * np.sin(phi)
    # zs = np.cos(theta)

    phi_deg = np.degrees(phi)
    theta_deg = np.degrees(theta)

    return phi_deg, theta_deg
    # TODO independent on voxlelsize (2016-techtest-rotate3d.ipynb)


def as_seeds_inds(seeds, datashape):
    shape = np.asarray(seeds).shape
    if datashape is not None and np.array_equal(shape, datashape):
        seeds_inds = np.nonzero(seeds)
    else:
        seeds_inds = seeds
    return seeds_inds


def resize_to_shape(data, shape, zoom=None, mode='reflect', order=0):
    """Resize input (gray-scale or color) data to specific shape.

    :param data: input 2d or 3d array-like data with shape (rows, cols[, …][, color_dim]),
    :param shape: shape of output data. Dimension should be N-1 for color images.
    :param zoom: zoom is used for back compatibility
    :mode: default is 'nearest'
    """
    # @TODO remove old code in except part

    if np.array_equal(data.shape, shape) or ((len(data.shape) ==  len(shape) + 1) and (np.array_equal(data.shape[:-1], shape))):
        # if output shape is same (for color the color number dimension is not checked)
        return data

    try:
        # rint 'pred vyjimkou'
        # aise Exception ('test without skimage')
        # rint 'za vyjimkou'
        import skimage
        import skimage.transform
        # Now we need reshape  seeds and segmentation to original size

        segm_orig_scale = skimage.transform.resize(
            data, shape, order=order,
            preserve_range=True,
            mode=mode,
        )

        segmentation = segm_orig_scale
        logger.debug('resize to orig with skimage')
    except ImportError:
        DeprecationWarning("Resize by scipy will be removed in the future")
        mode = 'nearest'
        import scipy
        import scipy.ndimage
        dtype = data.dtype
        if zoom is None:
            zoom = np.asarray(data.shape).astype(np.double) / shape

        segm_orig_scale = scipy.ndimage.zoom(
            data,
            1.0 / zoom,
            mode=mode,
            order=order
        ).astype(dtype)
        logger.debug('resize to orig with scipy.ndimage')

        segmentation = fit_to_shape(segm_orig_scale, shape, dtype)
        del segm_orig_scale
    return segmentation


def fit_to_shape(segm_orig_scale, shape, dtype):
    # @TODO odstranit hack pro oříznutí na stejnou velikost
    # v podstatě je to vyřešeno, ale nechalo by se to dělat elegantněji v zoom
    # tam je bohužel patrně bug
    # rint 'd3d ', self.data3d.shape
    # rint 's orig scale shape ', segm_orig_scale.shape
    shp = [
        np.min([segm_orig_scale.shape[0], shape[0]]),
        np.min([segm_orig_scale.shape[1], shape[1]]),
        np.min([segm_orig_scale.shape[2], shape[2]]),
    ]
    # elf.data3d = self.data3d[0:shp[0], 0:shp[1], 0:shp[2]]
    # mport ipdb; ipdb.set_trace() # BREAKPOINT

    segmentation = np.zeros(shape, dtype=dtype)
    segmentation[:shp[0], :shp[1], :shp[2]] = segm_orig_scale[:shp[0], :shp[1], :shp[2]]

    return segmentation


def calculate_new_shape(shape, voxelsize_mm, new_voxelsize_mm):
    if new_voxelsize_mm is 'orig':
        new_voxelsize_mm = np.asarray(voxelsize_mm)

    elif new_voxelsize_mm is 'orig*2':
        new_voxelsize_mm = np.asarray(voxelsize_mm) * 2
    elif new_voxelsize_mm is 'orig*4':
        new_voxelsize_mm = np.asarray(voxelsize_mm) * 4
    else:
        new_voxelsize_mm = np.asarray(new_voxelsize_mm)
        # vx_size = np.array(metadata['voxelsize_mm']) * 4

    zoom = voxelsize_mm / (1.0 * new_voxelsize_mm)
    # data3d_res = scipy.ndimage.zoom(
    #     data3d,
    #     zoom,
    #     mode=mode,
    #     order=order
    # ).astype(data3d.dtype)

    # probably better implementation
    if len(shape) == len(voxelsize_mm):
        new_shape = np.ceil(shape * zoom).astype(np.int)
    elif len(shape) == (len(voxelsize_mm) + 1):
        new_shape = np.ceil(shape[:-1] * zoom).astype(np.int)
    else:
        raise ValueError("Input shape is not compatible with given voxelsize_mm.")

    return new_shape


def resize_to_mm(data3d, voxelsize_mm, new_voxelsize_mm, mode='reflect', order=1):
    """
    Function can resize (grayscale or color) data3d or segmentation to specifed voxelsize_mm

    :param data3d: input 2d or 3d array-like data with shape (rows, cols[, …][, color_dim]),
    :param new_voxelsize_mm: requested voxelsize. List of 2 or 3 numbers. The color image is expected if
    the dim of voxelsize_mm is N-1 (where N is dimension of input data)
    Also string can be a used: 'orig', 'orig*2' and 'orig*4'.

    :param voxelsize_mm: size of voxel
    :param mode: default is 'edge'. Modes match the behaviour of numpy.pad
    """

    new_shape = calculate_new_shape(data3d.shape, voxelsize_mm, new_voxelsize_mm)

    import skimage.transform
    # Now we need reshape  seeds and segmentation to original size

    data3d_res2 = skimage.transform.resize(
        data3d, new_shape, order=order,
        mode=mode,
        preserve_range=True
    ).astype(data3d.dtype)

    return data3d_res2


def manualcrop(data):  # pragma: no cover

    try:
        from pysegbase import seed_editor_qt
    except:
        logger.warning("Deprecated of pyseg_base as submodule")
        import seed_editor_qt

    pyed = seed_editor_qt.QTSeedEditor(data, mode='crop')
    pyed.exec_()
    # pyed = sed3.sed3(data)
    # pyed.show()
    nzs = pyed.seeds.nonzero()
    crinfo = [
        [np.min(nzs[0]), np.max(nzs[0])],
        [np.min(nzs[1]), np.max(nzs[1])],
        [np.min(nzs[2]), np.max(nzs[2])],
    ]
    data = crop(data, crinfo)
    return data, crinfo


def crop(data, crinfo):
    """
    Crop the data.

    crop(data, crinfo)

    :param crinfo: min and max for each axis - [[minX, maxX], [minY, maxY], [minZ, maxZ]]

    """
    crinfo = fix_crinfo(crinfo)
    return data[
           __int_or_none(crinfo[0][0]):__int_or_none(crinfo[0][1]),
           __int_or_none(crinfo[1][0]):__int_or_none(crinfo[1][1]),
           __int_or_none(crinfo[2][0]):__int_or_none(crinfo[2][1])
           ]


def __int_or_none(number):
    if number is not None:
        number = int(number)
    return number


def combinecrinfo(crinfo1, crinfo2):
    """
    Combine two crinfos. First used is crinfo1, second used is crinfo2.
    """
    crinfo1 = fix_crinfo(crinfo1)
    crinfo2 = fix_crinfo(crinfo2)

    crinfo = [
        [crinfo1[0][0] + crinfo2[0][0], crinfo1[0][0] + crinfo2[0][1]],
        [crinfo1[1][0] + crinfo2[1][0], crinfo1[1][0] + crinfo2[1][1]],
        [crinfo1[2][0] + crinfo2[2][0], crinfo1[2][0] + crinfo2[2][1]]
    ]

    return crinfo


def extend_crinfo(crinfo, shape, margin):
    crinfo = fix_crinfo(crinfo, with_slices=True)
    d0 = max(0, crinfo[0].start - margin)
    u0 = min(shape[0], crinfo[0].stop + margin)
    slice_z = slice(d0, u0)
    d1 = max(0, crinfo[1].start - margin)
    u1 = min(shape[1], crinfo[1].stop + margin)
    slice_y = slice(d1, u1)
    d2 = max(0, crinfo[2].start - margin)
    u2 = min(shape[2], crinfo[2].stop + margin)
    slice_x = slice(d2, u2)
    crinfoo = (slice_z, slice_y, slice_x)
    return crinfoo



def uncrop(data, crinfo, orig_shape, resize=False, outside_mode="constant", cval=0):
    """
    Put some boundary to input image.


    :param data: input data
    :param crinfo: array with minimum and maximum index along each axis
        [[minX, maxX],[minY, maxY],[minZ, maxZ]]. If crinfo is None, the whole input image is placed into [0, 0, 0].
        If crinfo is just series of three numbers, it is used as an initial point for input image placement.
    :param orig_shape: shape of uncropped image
    :param resize: True or False (default). Usefull if the data.shape does not fit to crinfo shape.
    :param outside_mode: 'constant', 'nearest'
    :return:
    """

    if crinfo is None:
        crinfo = list(zip([0] * data.ndim, orig_shape))
    elif np.asarray(crinfo).size == data.ndim:
        crinfo = list(zip(crinfo, np.asarray(crinfo) + data.shape))

    crinfo = fix_crinfo(crinfo)
    data_out = np.ones(orig_shape, dtype=data.dtype) * cval

    # print 'uncrop ', crinfo
    # print orig_shape
    # print data.shape
    if resize:
        data = resize_to_shape(data, crinfo[:, 1] - crinfo[:, 0])

    startx = np.round(crinfo[0][0]).astype(int)
    starty = np.round(crinfo[1][0]).astype(int)
    startz = np.round(crinfo[2][0]).astype(int)

    data_out[
    # np.round(crinfo[0][0]).astype(int):np.round(crinfo[0][1]).astype(int)+1,
    # np.round(crinfo[1][0]).astype(int):np.round(crinfo[1][1]).astype(int)+1,
    # np.round(crinfo[2][0]).astype(int):np.round(crinfo[2][1]).astype(int)+1
    startx:startx + data.shape[0],
    starty:starty + data.shape[1],
    startz:startz + data.shape[2]
    ] = data

    if outside_mode == "nearest":
        # for ax in range(data.ndims):
        # ax = 0

        # copy border slice to pixels out of boundary - the higher part
        for ax in range(data.ndim):
            # the part under the crop
            start = np.round(crinfo[ax][0]).astype(int)
            slices = [slice(None), slice(None), slice(None)]
            slices[ax] = start
            repeated_slice = np.expand_dims(data_out[slices], ax)
            append_sz = start
            if append_sz > 0:
                tile0 = np.repeat(repeated_slice, append_sz, axis=ax)
                slices = [slice(None), slice(None), slice(None)]
                slices[ax] = slice(None, start)
                # data_out[start + data.shape[ax] : , :, :] = tile0
                data_out[slices] = tile0
                # plt.imshow(np.squeeze(repeated_slice))
                # plt.show()

            # the part over the crop
            start = np.round(crinfo[ax][0]).astype(int)
            slices = [slice(None), slice(None), slice(None)]
            slices[ax] = start + data.shape[ax] - 1
            repeated_slice = np.expand_dims(data_out[slices], ax)
            append_sz = data_out.shape[ax] - (start + data.shape[ax])
            if append_sz > 0:
                tile0 = np.repeat(repeated_slice, append_sz, axis=ax)
                slices = [slice(None), slice(None), slice(None)]
                slices[ax] = slice(start + data.shape[ax], None)
                # data_out[start + data.shape[ax] : , :, :] = tile0
                data_out[slices] = tile0
                # plt.imshow(np.squeeze(repeated_slice))
                # plt.show()

    return data_out


def fix_crinfo(crinfo, to='axis', with_slices=False):
    """
    Function recognize order of crinfo and convert it to proper format.
    """

    if type(crinfo[0]) is slice:
        if with_slices:
            return crinfo
        else:
            crinfo = [
                [crinfo[0].start, crinfo[0].stop],
                [crinfo[1].start, crinfo[1].stop],
                [crinfo[2].start, crinfo[2].stop],
            ]
    else:
        crinfo = np.asarray(crinfo)
        if crinfo.shape[0] == 2:
            crinfo = crinfo.T

        if with_slices:
            crinfo = (
                slice(crinfo[0][0], crinfo[0][1]),
                slice(crinfo[1][0], crinfo[1][1]),
                slice(crinfo[2][0], crinfo[2][1])
            )
        else:
            pass

    return crinfo



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
# import funkcí z jiného adresáře
import os.path

import unittest

import numpy as np
import os

# import io3d
import io3d.datasets
import imma.sparse as ima


class ImageManipulationTest(unittest.TestCase):
    interactivetTest = False

    # interactivetTest = True

    def test_store_to_SparseMatrix_and_back(self):
        data = np.zeros([4, 4, 4])
        data = np.zeros([4, 4, 4])
        data[1, 0, 3] = 1
        data[2, 1, 2] = 1
        data[0, 1, 3] = 2
        data[1, 2, 0] = 1
        data[2, 1, 1] = 3

        dataSM = ima.SparseMatrix(data)
        self.assertTrue(ima.isSparseMatrix(dataSM), "Check sparse matrix")

        data2 = dataSM.todense()
        self.assertTrue(np.all(data == data2))


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)

import numpy as np
import scipy

from . import segmentation_labels
ima = segmentation_labels

def select_labels(segmentation, labels, slab=None):
    """
    return ndimage with zeros and ones based on input labels

    :param segmentation: 3D ndimage
    :param labels: labels to select
    :param slab: dictionary{string_label: numeric_label}. Allow to use
    string labels if it is defined
    :return:
    """

    if slab is not None:
        labels = segmentation_labels.get_nlabels(slab, labels)

    if type(labels) not in (list, np.ndarray, set):
        labels = [labels]

    ds = np.zeros(segmentation.shape, np.bool)
    for lab in labels:
        dadd = (segmentation == lab)

        ds = ds | dadd
    if len(labels) == 0:
        logger.warning("Labels not found in slab.")

    return ds


def get_one_biggest_object(data):
    """ Return biggest object """
    lab, num = scipy.ndimage.label(data)
    # print ("bum = "+str(num))

    maxlab = max_area_index(lab, num)

    data = (lab == maxlab)
    return data


def max_area_index(labels, num=None):
    """
    Return index of maxmum labeled area
    """
    mx = 0
    mxi = -1
    un = np.unique(labels)
    # kick out zero
    un = un[un != 0]
    for l in un:
        mxtmp = np.sum(labels == l)
        if mxtmp > mx:
            mx = mxtmp
            mxi = l

    return mxi


def max_area_index2(labels, num):
    """
    Return index of maxmum labeled area. Old implementation. Slower.
    """
    DeprecationWarning("Function will be removed in future")
    mx = 0
    mxi = -1
    for l in range(1, num + 1):
        mxtmp = np.sum(labels == l)
        if mxtmp > mx:
            mx = mxtmp
            mxi = l

    return mxi




def select_objects_by_seeds(binar_data, seeds, ignore_background_seeds=True, background_label=0):
    """
    Get N biggest objects from the selection or the object with seed.

    :param binar_data:  binar ndarray
    :param seeds: ndarray. Objects on non zero positions are returned
    :return:
    """

    labeled_data, length = scipy.ndimage.label(binar_data)
    selected_labels = list(np.unique(labeled_data[seeds > 0]))
    # selected_labels.pop(0)
    # pop the background label
    output = np.zeros_like(binar_data)
    for label in selected_labels:
        selection = labeled_data == label
        # copy from input image to output. If there will be seeds in background, the 0 is copied
        if ignore_background_seeds and (binar_data[selection][0] == background_label):
            pass
        else:
            # output[selection] = binar_data[selection]
            output[selection] = 1
    # import sed3
    # ed =sed3.sed3(labeled_data, contour=output, seeds=seeds)
    # ed.show()
    return output



def squeeze_labels(segmentation, try_inplace=True):
    """
    Squeeze all labels to int numbers starting from 1

    :param segmentation: labeled image
    :param try_inplace: try to compute inplace
    :return:
    """
    un = np.unique(segmentation)

    if try_inplace:
        inplace_possible = True
        for new_level, level in enumerate(un):
            # if i == level:
            #     continue
            if (new_level > level) and (new_level in un):
                # we will rewrite old
                inplace_possible = False
                break
    else:
        inplace_possible = False

    if inplace_possible:
        output = segmentation
    else:
        import copy
        output = copy.copy(segmentation)

    for new_level, level in enumerate(un):
        output[segmentation == level] = new_level

    return output


def distance_segmentation(seeds, method="edt"):
    """
    Separates space based on distance to seeds.
    :param seeds: ndarray with zeros for background and labeled seeds.
    :param method: scipy.ndimage.distance_transform function.
    The `distance_transform_edt` is used if is set to "edt"
    :return:
    """
    if method is "edt":
        dst_transform = scipy.ndimage.distance_transform_edt
    else:
        dst_transform = method

    inds = dst_transform(seeds == 0, return_indices=True, return_distances=False)

    lin_inds = []
    for one_ax in inds:
        lin_inds.append(one_ax.ravel())

    segm = seeds[lin_inds].reshape(seeds.shape)
    return segm

    pass

def crinfo_from_specific_data(data, margin=0, with_slices=False):
    """
    Create crinfo of minimum orthogonal nonzero block in input data.

    :param data: input data
    :param margin: add margin to minimum block
    :return:
    """
    # hledáme automatický ořez, nonzero dá indexy
    logger.debug('crinfo')
    logger.debug(str(margin))
    nzi = np.nonzero(data)
    logger.debug(str(nzi))

    if np.isscalar(margin):
        margin = [margin] * 3

    x1 = np.min(nzi[0]) - margin[0]
    x2 = np.max(nzi[0]) + margin[0] + 1
    y1 = np.min(nzi[1]) - margin[0]
    y2 = np.max(nzi[1]) + margin[0] + 1
    z1 = np.min(nzi[2]) - margin[0]
    z2 = np.max(nzi[2]) + margin[0] + 1

    # ošetření mezí polí
    if x1 < 0:
        x1 = 0
    if y1 < 0:
        y1 = 0
    if z1 < 0:
        z1 = 0

    if x2 > data.shape[0]:
        x2 = data.shape[0] - 1
    if y2 > data.shape[1]:
        y2 = data.shape[1] - 1
    if z2 > data.shape[2]:
        z2 = data.shape[2] - 1

    # ořez
    if with_slices:
        crinfo = (slice(x1, x2), slice(y1, y2), slice(z1, z2))
    else:
        crinfo = [[x1, x2], [y1, y2], [z1, z2]]
    return crinfo


def unique_labels_by_seeds(labeled, seeds, ignored_seeds=0):
    """
    Get labels on positions of seeds.
    :param labeled: labeled ndimage
    :param seeds: ndimage with seeds
    :param ignored_seeds: int or list with labels to ignore, default ignored seed is 0
    :return: Dictionary. Keys are seeds, values are unique labels corresponding to this seeds.
    """

    if type(ignored_seeds) is not list:
        ignored_seeds = [ignored_seeds]
    # un = np.unique(seeds)
    # output = [None] * len(un)
    output = {}
    for seed in np.unique(seeds):
        if seed not in ignored_seeds:
            output[seed] = np.unique(labeled[seeds == seed])

    return output




#! /usr/bin/env python
# -*- coding: utf-8 -*-
# vim:fenc=utf-8

import logging

logger = logging.getLogger(__name__)
import numpy as np

import unittest
from collections import OrderedDict
from imma import obj


class ObjTestCase(unittest.TestCase):
    def generate_dict_data(self):
        data = {
            'a': 1,
            'b': 2,
            'c': {
                'aa': 11,
                'bb': 22,
                'cc': {
                    'aaa': 111
                }
            }
        }
        return data

    def test_get_standard_arguments_obj(self):
        from collections import OrderedDict
        args, kwargs = obj.get_default_args(Foo)
        self.assertEqual(type(kwargs), OrderedDict)
        self.assertIn("first", kwargs)
        self.assertEqual(kwargs["first"], None)
        self.assertEqual(kwargs["second"], 5)
        self.assertEqual(kwargs["third"], [])

    # @unittest.skip("Waiting for implementation")
    def test_get_standard_arguments_with_position_arg(self):
        from collections import OrderedDict
        args, kwargs = obj.get_default_args(Bar)
        self.assertEqual(type(kwargs), OrderedDict)
        self.assertIn("first", kwargs)
        self.assertEqual(kwargs["first"], None)
        self.assertEqual(kwargs["second"], 5)
        self.assertEqual(kwargs["third"], [])

        self.assertIn("zero" ,args)

    def test_get_standard_arguments_function(self):
        from collections import OrderedDict
        args, kwargs = obj.get_default_args(foo)
        self.assertEqual(type(kwargs), OrderedDict)
        self.assertIn("first", kwargs)
        self.assertEqual(kwargs["first"], None)
        self.assertEqual(kwargs["second"], 5)
        self.assertEqual(kwargs["third"], [])
        self.assertIn("zero", args)


class Foo:
    def __init__(self, first=None, second=5, third=[]):
        pass


class Bar:
    def __init__(self, zero, first=None, second=5, third=[]):
        pass

def foo(zero, first=None, second=5, third=[]):
    pass

def main():
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
# import funkcí z jiného adresáře
import os.path

import unittest

import numpy as np
import os

# import io3d
import io3d.datasets
import imma.image as ima
import imma.segmentation_labels as imsg
import imma.labeled as imlb



class ImageManipulationTest(unittest.TestCase):

    def test_resize_to_shape(self):
        data = np.random.rand(3, 4, 5)
        new_shape = [5, 6, 6]
        data_out = ima.resize_to_shape(data, new_shape)
        # print data_out.shape
        # print data
        # print data_out
        self.assertEquals(new_shape[0], data_out.shape[0])
        self.assertEquals(new_shape[1], data_out.shape[1])
        self.assertEquals(new_shape[2], data_out.shape[2])

    def test_resize_color_to_shape(self):
        rgbimg = make_color_image()
        # plt.imshow(rgbimg)

        new_shape = [99, 99]
        imgres = ima.resize_to_shape(rgbimg, new_shape)

        # plt.imshow(imgres)
        # plt.colorbar()

        red = imgres[70, 20]
        blue = imgres[20, 70]
        green = imgres[70, 70]
        white = imgres[10, 10]

        # display(red)V
        # display(green)
        # display(blue)
        self.assertTrue(np.array_equal(red, [255, 0, 0]))
        self.assertTrue(np.array_equal(green, [0, 255, 0]))
        self.assertTrue(np.array_equal(blue, [0, 0, 255]))
        self.assertTrue(np.array_equal(white, [255, 255, 255]))

    def test_resize_to_mm(self):
        data = np.random.rand(3, 4, 6)
        voxelsize_mm = [2, 3, 1]
        new_voxelsize_mm = [1, 3, 2]
        expected_shape = [6, 4, 3]
        data_out = ima.resize_to_mm(data, voxelsize_mm, new_voxelsize_mm)
        # print(data_out.shape)
        # print data
        # print data_out
        self.assertEquals(expected_shape[0], data_out.shape[0])
        self.assertEquals(expected_shape[1], data_out.shape[1])
        self.assertEquals(expected_shape[2], data_out.shape[2])

    def test_resize_color_to_mm(self):
        rgbimg = make_color_image()
        # plt.imshow(rgbimg)


        imgres = ima.resize_to_mm(rgbimg, voxelsize_mm=[1, 1], new_voxelsize_mm=[.1001, .1001])

        # plt.imshow(imgres)
        # plt.colorbar()

        red = imgres[70, 20]
        blue = imgres[20, 70]
        green = imgres[70, 70]
        white = imgres[10, 10]

        # display(red)V
        # display(green)
        # display(blue)
        self.assertTrue(np.array_equal(red, [255, 0, 0]))
        self.assertTrue(np.array_equal(green, [0, 255, 0]))
        self.assertTrue(np.array_equal(blue, [0, 0, 255]))
        self.assertTrue(np.array_equal(white, [255, 255, 255]))



    def test_crop_and_uncrop(self):
        shape = [10, 10, 5]
        img_in = np.random.random(shape)

        crinfo = [[2, 8], [3, 9], [2, 5]]

        img_cropped = ima.crop(img_in, crinfo)

        img_uncropped = ima.uncrop(img_cropped, crinfo, shape)

        self.assertTrue(img_uncropped[4, 4, 3] == img_in[4, 4, 3])


    def test_extend_crinfo(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        crinfo1 = imlb.crinfo_from_specific_data(segmentation, [5])
        crinfo2 = ima.extend_crinfo(crinfo1, data3d.shape, 3)
        self.assertEqual(type(data3d[crinfo2]), np.ndarray, "We are able to use slices in data with extended crinfo.")

    def test_multiple_crop_and_uncrop(self):
        """
        test combination of multiple crop
        """

        shape = [10, 10, 5]
        img_in = np.random.random(shape)

        crinfo1 = [[2, 8], [3, 9], [2, 5]]
        crinfo2 = [[2, 5], [1, 4], [1, 2]]

        img_cropped = ima.crop(img_in, crinfo1)
        img_cropped = ima.crop(img_cropped, crinfo2)

        crinfo_combined = ima.combinecrinfo(crinfo1, crinfo2)

        img_uncropped = ima.uncrop(img_cropped, crinfo_combined, shape)

        self.assertTrue(img_uncropped[4, 4, 3] == img_in[4, 4, 3])
        self.assertEquals(img_in.shape, img_uncropped.shape)

    @unittest.skip("crinfo_combine should be tested in different way")
    def test_random_multiple_crop_and_uncrop(self):
        """
        test combination of multiple crop
        """

        shape = np.random.randint(10, 30, 3)
        # shape = [10, 10, 5]
        img_in = np.random.random(shape)

        crinfo1 = [
            sorted(np.random.randint(0, shape[0], 2)),
            sorted(np.random.randint(0, shape[1], 2)),
            sorted(np.random.randint(0, shape[2], 2))
        ]
        crinfo2 = [
            sorted(np.random.randint(0, shape[0], 2)),
            sorted(np.random.randint(0, shape[1], 2)),
            sorted(np.random.randint(0, shape[2], 2))
        ]

        img_cropped = ima.crop(img_in, crinfo1)
        img_cropped = ima.crop(img_cropped, crinfo2)

        crinfo_combined = ima.combinecrinfo(crinfo1, crinfo2)

        img_uncropped = ima.uncrop(img_cropped, crinfo_combined, shape)
        logger.debug("shape " + str(shape))
        logger.debug("crinfo_combined " + str(crinfo_combined))
        logger.debug("img_cropped.shape" + str(img_cropped.shape))
        logger.debug("img_uncropped.shape" + str(img_uncropped.shape))

        self.assertEquals(img_in.shape, img_uncropped.shape)
        # sonda indexes inside cropped area
        # cr_com = np.asarray(crinfo_combined)
        # if np.all((cr_com[:, 1] - cr_com[:, 0]) > 1):
        if np.all(img_cropped.shape > 1):
            # sometimes the combination of crinfo has zero size in one dimension
            sonda = np.array([
                np.random.randint(crinfo_combined[0][0], crinfo_combined[0][1] - 1),
                np.random.randint(crinfo_combined[1][0], crinfo_combined[1][1] - 1),
                np.random.randint(crinfo_combined[2][0], crinfo_combined[2][1] - 1),
            ])
            sonda_intensity_uncropped = img_uncropped[sonda[0], sonda[1], sonda[2]]
            sonda_intensity_in = img_in[sonda[0], sonda[1], sonda[2]]
            self.assertEquals(sonda_intensity_in, sonda_intensity_uncropped)

    def test_resize_to_shape(self):
        data = np.random.rand(3, 4, 5)
        new_shape = [5, 6, 6]
        data_out = ima.resize_to_shape(data, new_shape)
        # self.assertCountEqual(new_shape, data_out.shape)
        self.assertEqual(new_shape[0], data_out.shape[0])
        self.assertEqual(new_shape[1], data_out.shape[1])
        self.assertEqual(new_shape[2], data_out.shape[2])

    def test_resize_to_shape_no_new_unique_values(self):
        data = np.zeros([10, 15, 12])
        value1 = 1
        value2 = 2
        data[:5, :7, :6] = value1
        data[-5:, :7, :6] = value2

        expected_shape = [15, 15, 15]
        resized = ima.resize_to_shape(data, expected_shape)
        unique = np.unique(resized)

        self.assertEqual(resized.shape[0], expected_shape[0])
        self.assertEqual(resized.shape[1], expected_shape[1])
        self.assertEqual(resized.shape[2], expected_shape[2])
        self.assertEqual(resized[1, 1, 1], value1)
        self.assertEqual(resized[-2, 1, 1], value2)
        self.assertEqual(len(unique), 3)
        self.assertEqual(unique[0], 0)
        self.assertEqual(unique[1], 1)
        self.assertEqual(unique[2], 2)

    def test_fit_to_shape(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        segmentation = datap["segmentation"]
        #orig shape 100x100x100
        shape = [95, 117, 100]
        new_seg = ima.fit_to_shape(segmentation, shape, segmentation.dtype)
        self.assertTrue(np.array_equal(new_seg.shape, shape))

    def test_fix_crinfo(self):
        crinfo = [[10, 15], [30, 40], [1, 50]]
        cri_fixed = ima.fix_crinfo(crinfo)

        # print crinfo
        # print cri_fixed

        self.assertTrue(cri_fixed[1, 1] == 40)
        self.assertTrue(cri_fixed[2, 1] == 50)

    def test_multiple_crop_and_uncrop_nearest_outside(self):
        """
        test combination of multiple crop
        """

        shape = [10, 11, 5]
        img_in = np.random.random(shape)

        crinfo1 = [[2, 8], [3, 9], [2, 5]]
        # crinfo2 = [[2, 5], [1, 5], [1, 2]]

        img_cropped = ima.crop(img_in, crinfo1)
        # img_cropped = imma.crop(img_cropped, crinfo2)

        # crinfo_combined = imma.combinecrinfo(crinfo1, crinfo2)

        img_uncropped = ima.uncrop(img_cropped, crinfo1, shape, outside_mode="nearest")

        # import sed3
        # ed = sed3.sed3(img_uncropped)
        # ed.show()
        self.assertTrue(img_uncropped[4, 4, 3] == img_in[4, 4, 3])

        self.assertTrue(img_uncropped[crinfo1[0][0], 5, 3] == img_uncropped[0, 5, 3], msg="pixels under crop")
        self.assertTrue(img_uncropped[5, crinfo1[1][0], 3] == img_uncropped[5, 0, 3], msg="pixels under crop")
        self.assertTrue(img_uncropped[7, 3, crinfo1[2][0]] == img_uncropped[7, 3, 0], msg="pixels under crop")

        self.assertTrue(img_uncropped[crinfo1[0][1] - 1, 5, 3] == img_uncropped[-1, 5, 3], msg="pixels over crop")
        self.assertTrue(img_uncropped[5, crinfo1[1][1] - 1, 3] == img_uncropped[5, -1, 3], msg="pixels over crop")
        self.assertTrue(img_uncropped[7, 3, crinfo1[2][1] - 1] == img_uncropped[7, 3, -1], msg="pixels over crop")

        # self.assertTrue(img_uncropped[crinfo1[0][1], 5 , 3] == img_uncropped[0, 5, 3], msg="pixels over crop")
        # self.assertTrue(img_uncropped[crinfo1[1][1], 5 , 3] == img_uncropped[1, 5, 3], msg="pixels over crop")
        # self.assertTrue(img_uncropped[crinfo1[2][1], 5 , 3] == img_uncropped[2, 5, 3], msg="pixels over crop")
        self.assertEquals(img_in.shape, img_uncropped.shape)

    def test_uncrop_with_none_crinfo(self):
        shape = [10, 10, 5]
        orig_shape = [15, 13, 7]
        img_in = np.random.random(shape)

        img_uncropped = ima.uncrop(img_in, crinfo=None, orig_shape=orig_shape)

        self.assertTrue(img_uncropped[-1, -1, -1] == 0)
        self.assertTrue(img_uncropped[4, 4, 3] == img_in[4, 4, 3])

    def test_uncrop_with_start_point_crinfo(self):
        shape = [10, 10, 5]
        orig_shape = [15, 13, 7]
        img_in = np.random.random(shape)
        crinfo = [5, 2, 1]

        img_uncropped = ima.uncrop(img_in, crinfo=crinfo, orig_shape=orig_shape)

        self.assertTrue(img_uncropped[-1, -1, -1] == 0)
        self.assertTrue(img_uncropped[4 + 5, 4 + 2, 3 + 1] == img_in[4, 4, 3])


    def test_resize_to_mm(self):
        data = np.random.rand(3, 4, 5)
        voxelsize_mm = [2, 3, 1]
        new_voxelsize_mm = [1, 3, 2]
        expected_shape = [6, 4, 3]
        data_out = ima.resize_to_mm(data, voxelsize_mm, new_voxelsize_mm)
        self.assertEqual(expected_shape[0], data_out.shape[0])
        self.assertEqual(expected_shape[1], data_out.shape[1])
        self.assertEqual(expected_shape[2], data_out.shape[2])
        # self.assertCountEqual(expected_shape, data_out.shape)

    def test_rotate(self):
        datap = io3d.datasets.generate_abdominal()
        data3d = datap["data3d"]
        phi_deg, theta_deg = ima.random_rotate_paramteres()
        data3d_rot = ima.rotate(data3d, phi_deg, theta_deg)
        # import sed3
        # ed = sed3.sed3(data3d)
        # ed.show()
        # import sed3
        # ed = sed3.sed3(data3d_rot, contour=(data3d_rot < 0))
        # ed.show()
        self.assertLessEqual(np.min(data3d), np.min(data3d_rot))
        self.assertGreaterEqual(np.max(data3d), np.max(data3d_rot))


def make_color_image():
    rgbimg = np.zeros([10, 10, 3], dtype=np.uint8)
    rgbimg[:3, :3, :] = 255
    rgbimg[5:, :5, 0] = 255
    rgbimg[5:, 5:, 1] = 255
    rgbimg[:5, 5:, 2] = 255
    return rgbimg


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)

import unittest
import numpy as np

import imma.measure


class MeasurementTest(unittest.TestCase):

    def test_neigh_matrix(self):
        data = np.zeros((3, 4), dtype=int)
        data[:3, 1] = 1
        data[2:, 2] = 2
        data[0, 1:] = 3

        import imma.measure
        nbm = imma.measure.cooccurrence_matrix(data)
        nbm[1][0]
        self.assertEqual(nbm[1][0], nbm[0][1])
        self.assertEqual(nbm[3][0], nbm[0][3])

    def test_neigh_matrix_obj(self):
        data = np.zeros((3, 4), dtype=int)
        data[:3, 1] = 1
        data[2:, 2] = 2
        data[0, 1:] = 3
        #
        nbm = imma.measure.CooccurrenceMatrix(data)
        ndn = nbm.to_ndarray()
        sum = np.sum(ndn)
        sh = data.shape
        number_of_vertical_edges = 2 * 4
        number_of_horizontal_edges = 3 * 3
        self.assertEqual(sum, (number_of_horizontal_edges + number_of_vertical_edges) * 2)
        keys = nbm.keys()
        self.assertIn(0, keys)
        self.assertIn(1, keys)
        self.assertIn(2, keys)

        # print(nbm)
        # print data_out.shape
        # print data
        # print data_out
        # self.assertEquals(new_shape[0], data_out.shape[0])
        # self.assertEquals(new_shape[1], data_out.shape[1])
        # self.assertEquals(new_shape[2], data_out.shape[2])

    def test_neigh_matrix_obj_with_high_number(self):
        data = np.zeros((3, 4), dtype=int)
        data[:3, 1] = 1
        data[2:, 2] = 2
        data[0, 1:] = 6
        #
        nbm = imma.measure.CooccurrenceMatrix(data)
        ndn = nbm.to_ndarray()
        sum = np.sum(ndn)
        sh = data.shape
        number_of_vertical_edges = 2 * 4
        number_of_horizontal_edges = 3 * 3
        self.assertEqual(sum, (number_of_horizontal_edges + number_of_vertical_edges) * 2)
        keys = nbm.keys()
        self.assertIn(0, keys)
        self.assertIn(1, keys)
        self.assertIn(6, keys)

        inv_keys = nbm.inv_keys()
        key_in_table = inv_keys[6]
        six2zero_0 = ndn[inv_keys[6], inv_keys[0]]
        six2zero_1 = nbm.get(6, 0)
        self.assertEqual(six2zero_0, six2zero_1, "# 6 is neighboor of 0 ")

    def test_neigh_matrix_obj_not_return_counts(self):
        data = np.zeros((3, 4), dtype=int)
        data[:3, 1] = 1
        data[2:, 2] = 2
        data[0, 1:] = 3
        #
        nbm = imma.measure.CooccurrenceMatrix(data, return_counts=False)
        ndn = nbm.to_ndarray()
        sum = np.sum(ndn)
        self.assertLessEqual(np.max(ndn), 2)
        sh = data.shape
        # number_of_vertical_edges = 2 * 4
        # number_of_horizontal_edges = 3 * 3
        # self.assertEqual(sum, (number_of_horizontal_edges + number_of_vertical_edges) * 2)
        keys = nbm.keys()
        self.assertIn(0, keys)
        self.assertIn(1, keys)
        self.assertIn(2, keys)

    def test_element_neighboors(self):
        data = np.zeros((1, 3, 4), dtype=int)
        data[0, :3, 1] = 1
        data[0, 2:, 2] = 2
        data[0, 0, 1:] = 6
        # neighb, bbox = imma.measure.element_neighbors(data, 1)
        neighbs = imma.measure.neighbors_list(data)
        print(neighbs)

        self.assertTrue(np.array_equal(neighbs[1], [0, 2, 6]), "Label 1 is negbor to [0, 2, 6]")
        self.assertTrue(np.array_equal(neighbs[2], [0, 1]), "Label 2 is negbor to [0, 1]")
        self.assertTrue(neighbs[3] is None, "Label 4 has no neighboor")
        self.assertTrue(np.array_equal(neighbs[6], [0, 1]), "Label 6 is negbor to [0, 1]")

    def test_element_neighboors_with_selected_labels(self):
        data = np.zeros((1, 3, 4), dtype=int)
        data[0, :3, 1] = 1
        data[0, 2:, 2] = 2
        data[0, 0, 1:] = 6
        # neighb, bbox = imma.measure.element_neighbors(data, 1)
        neighbs = imma.measure.neighbors_list(data, labels=[2, 6])
        print(neighbs)

        # self.assertTrue(np.array_equal(neighbs[1], [0, 2, 6]), "Label 1 is negbor to [0, 2, 6]")
        self.assertTrue(np.array_equal(neighbs[0], [0, 1]), "Label 2 is negbor to [0, 1]")
        # self.assertTrue(neighbs[3] is None, "Label 4 has no neighboor")
        self.assertTrue(np.array_equal(neighbs[1], [0, 1]), "Label 6 is negbor to [0, 1]")

    def test_element_neighboors_with_exclude(self):
        data = np.zeros((1, 3, 4), dtype=int)
        data[0, :3, 1] = 1
        data[0, 2:, 2] = 2
        data[0, 0, 1:] = 6
        # neighb, bbox = imma.measure.element_neighbors(data, 1)
        neighbs = imma.measure.neighbors_list(data, exclude=[6])
        print(neighbs)

        self.assertTrue(np.array_equal(neighbs[1], [0, 2]), "Label 1 is negbor to [0, 2, 6]")
        self.assertTrue(np.array_equal(neighbs[2], [0, 1]), "Label 2 is negbor to [0, 1]")
        self.assertTrue(neighbs[4] is None, "Label 4 has no neighboor")
        self.assertTrue(np.array_equal(neighbs[6], [0, 1]), "Label 6 is negbor to [0, 1]")

    def test_connected_elements(self):
        data = np.zeros((1, 3, 4), dtype=int)
        data[0, :3, 1] = 1
        data[0, 2:, 2] = 2
        data[0, 0, 1:] = 6
        # neighb, bbox = imma.measure.element_neighbors(data, 1)
        neighbs = imma.measure.neighbors_list(data)
        connected  = imma.measure.get_connected_labels(neighbs, 6, ignore_labels=[0])
        print(neighbs)


        # self.assertTrue(np.array_equal(neighbs[1], [0, 2, 6]), "Label 1 is negbor to [0, 2, 6]")
        self.assertTrue(np.array_equal(neighbs[2], [0, 1]), "Label 2 is negbor to [0, 1]")
        # self.assertTrue(neighbs[3] is None, "Label 4 has no neighboor")
        self.assertTrue(np.array_equal(neighbs[6], [0, 1]), "Label 6 is negbor to [0, 1]")
        self.assertTrue(np.array_equal(list(connected), [1, 2, 6]), "All connected")

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)
import unittest
import numpy as np
import imma.geometry3d as g3


class GeometryTestCase(unittest.TestCase):
    def test_collision_model_no_collision(self):
        cm = g3.CollisionModelSpheres(areasize=[150, 151, 155])
        distance = 30
        pt1 = [20, 20, 20]
        pt2 = [20, 20, 60]
        pt3 = [20, 20 + distance, 20]
        pt4 = [20, 20 + distance, 60]
        r1 = 10
        r2 = 10
        collision1 = cm.add_cylinder_if_no_collision(pt1, pt2, radius=r1)
        collision2 = cm.add_cylinder_if_no_collision(pt3, pt4, radius=r2)

        self.assertEqual(collision2, False)

    def test_collision_model_collision(self):
        cm = g3.CollisionModelSpheres(areasize=[150, 151, 155])
        distance = 15
        pt1 = [20, 20, 20]
        pt2 = [20, 20, 60]
        pt3 = [20, 20 + distance, 20]
        pt4 = [20, 20 + distance, 60]
        r1 = 10
        r2 = 10
        collision1 = cm.add_cylinder_if_no_collision(pt1, pt2, radius=r1)
        collision2 = cm.add_cylinder_if_no_collision(pt3, pt4, radius=r2)

        self.assertEqual(collision2, True)

    def test_collision_model_out_of_area(self):
        """
        Cylinder end point should be in safe distance from boundary
        :return:
        """
        cm = g3.CollisionModelSpheres(areasize=[150, 151, 65])
        distance = 25
        pt1 = [20, 20, 20]
        pt2 = [20, 20, 60]
        pt3 = [20, 20 + distance, 20]
        pt4 = [20, 20 + distance, 60]
        r1 = 10
        r2 = 10
        collision1 = cm.add_cylinder_if_no_collision(pt1, pt2, radius=r1)
        collision2 = cm.add_cylinder_if_no_collision(pt3, pt4, radius=r2)

        self.assertEqual(collision2, True)

    def test_collision_model_n_closest(self):
        cm = g3.CollisionModelSpheres(areasize=[150, 151, 155])
        distance = 30
        pt1 = [20, 20, 20]
        pt2 = [20, 20, 60]
        pt3 = [20, 20 + distance, 20]
        pt4 = [20, 20 + distance, 60]
        r1 = 10
        r2 = 10
        collision1 = cm.add_cylinder_if_no_collision(pt1, pt2, radius=r1)
        nodes, indexes, distances = cm.n_closest_points([25, 35, 41], 3)

    def test_bbox_collision(self):
        bb1 = [[10, 20], [10, 20], [10, 20]]
        bb2 = [[15, 30], [15, 30], [15, 30]]
        bb3 = [[15, 30], [15, 30], [25, 30]]
        bb4 = [[5, 35], [5, 35], [15, 18]]
        params = [
            [bb1, bb2, True],
            [bb1, bb3, False],
            [bb1, bb4, True],
        ]
        for param in params:
            # out = g3.check_Collision(param[0], param[1])
            out = g3.bbox_collision(param[0], param[1])
            self.assertEqual(out, param[2])

            # g3.bbox_collision(bb1, bb2)

    def test_get_bbox(self):
        points = [[10, 20, 0], [30, 40, 0]]
        bbox_expected = np.asarray([[5, 35], [15, 45], [-5, 5]])

        bbox = g3.get_bbox(points, margin=5)
        err = np.sum((bbox - bbox_expected) ** 2)
        self.assertAlmostEquals(err, 0)

    def test_dist_between_lines(self):
        # TODO
        a0 = [0, 0, 0]
        a1 = [0, 0, 1]
        b0 = [5, 0, 1]
        b1 = [8, -3, -5]

        pa1, pb1, dist1 = g3.closest_distance_between_lines(a0, a1, b0, b1)
        pa2, pb2, dist2 = g3.closest_distance_between_lines(a0, a1, b0, b1, clamp_all=True)

        # self.assertAlmostEquals(dist, 0

    def test_get_line_nodes(self):
        nodes = g3.get_points_in_line_segment([10, 13, 22], [1, 13, 22], 3)
        expected_x = [10, 7, 4, 1]

        self.assertAlmostEqual(nodes[1][0], expected_x[1])
        self.assertAlmostEqual(nodes[2][0], expected_x[2])
        self.assertAlmostEqual(nodes[3][0], expected_x[3])

    def test_dist_between_lines_problematic_case(self):
        # TODO
        a0 = [15.66021867, 18.12350593, 14.89322993]
        a1 = [15.66021867, 17.10675439, 13.87647838]
        b0 = [16.05191401, 21.03424487, 10.59026252]
        b1 = [16.05191401, 13.6576145, 3.21363215]

        pa1, pb1, dist1 = g3.closest_distance_between_lines(a0, a1, b0, b1)
        pa2, pb2, dist2 = g3.closest_distance_between_lines(a0, a1, b0, b1, clamp_all=True)

    def test_dist_between_paralel_lines(self):
        a0 = [0, 0, 0]
        a1 = [0, 0, 1]
        b0 = [1, 0, 0]
        b1 = [1, 0, 1]

        pa, pb, dist = g3.closest_distance_between_lines(a0, a1, b0, b1)

        self.assertAlmostEquals(dist, 1.0)

    def test_point_in_plane(self):
        pl_pt = [0, 0, 0]
        pl_vect = [1, 0, 0]

        pts = np.asarray([
            [0, 0, 1],
            [0, -1, 1],
            [1, 1, 1],
            [1, 15, 13],
            [-1, -1, 1],
            [-8, -1, 1]
        ])

        retval_expected = [
            0,
            0,
            1,
            1,
            -1,
            -1,
        ]
        retval = g3.point_and_plane_pose(pl_pt, pl_vect, pts)
        retval = np.sign(retval)
        err = np.sum((retval - retval_expected) ** 2)

        self.assertAlmostEqual(err, 0)

    def test_cylinder_collision_model_collision(self):
        cm = g3.CollisionModelSpheres(areasize=[150, 151, 155])
        distance = 25
        pt1 = [20, 20, 20]
        pt2 = [20, 20, 60]
        pt3 = [20, 20 + distance, 20]
        pt4 = [20, 20 + distance, 60]
        ptC1 = [0, 0, 0]
        ptC2 = [0, 60, 0]
        rA = 10
        rB = 10
        rC = 15
        cylA = g3.CylinderObject(pt1, pt2, radius=rA)
        cylB = g3.CylinderObject(pt3, pt4, radius=rB)
        cylC = g3.CylinderObject(ptC1, ptC2, radius=rC)
        collision1 = cylA.collision(cylB)
        collision2 = cylA.collision(cylC)

        self.assertEqual(collision1, False)
        self.assertEqual(collision2, False)

    def test_bbox_corners(self):
        bbox = [[5, 10], [13, 17], [16, 18]]
        points = g3.get_bbox_corners(bbox)

        expected_point = np.array([10, 13, 16])
        found = False
        for point in points:
            err = np.sum((point - expected_point) ** 2)
            if err == 0:
                found = True
                break

        self.assertTrue(found)

    def test_cylinder_collision_model_out_of_area(self):
        """
        Cylinder end point should be in safe distance from boundary
        :return:
        """
        cm = g3.CollisionModelSpheres(areasize=[150, 151, 65])
        distance = 25
        pt1 = [20, 20, 20]
        pt2 = [20, 20, 60]
        pt3 = [20, 20 + distance, 20]
        pt4 = [20, 20 + distance, 60]
        r1 = 10
        r2 = 10
        collision1 = cm.add_cylinder_if_no_collision(pt1, pt2, radius=r1)
        collision2 = cm.add_cylinder_if_no_collision(pt3, pt4, radius=r2)

        self.assertEqual(collision2, True)

    def test_cylinder_collision_model_dist_problem(self):
        """
        check where is problem in two lines dist function
        :return:
        """

        cm = g3.CollisionModelSpheres(areasize=[150, 151, 155])

        distance = 25
        pt1 = [20, 20, 20]
        pt2 = [20, 20, 60]
        pt3 = [20, 20 + distance, 20]
        pt4 = [20, 20 + distance, 60]
        ptC1 = [0, 0, 0]
        ptC2 = [0, 60, 0]
        rA = 10
        rB = 10
        rC = 15
        cylA = g3.CylinderObject(
            [8.2932388, -0.65185999, 9.54170155],
            [8.2932388, 11.23408448, 21.42764602],
            radius=5.691834898023455)
        cylB = g3.CylinderObject(
            [2.76365903, 0.86076129, 4.30361747],
            [2.76365903, 7.00253318, 10.44538936],
            radius=7.2248666037231875
        )
        collision1 = cylA.collision(cylB)

        self.assertEqual(collision1, True)

    def test_3d_rotation(self):
        pass

    def test_polar(self):
        ptA = [[1, 2, 3]]
        dist, angle = g3.z2polar(ptA)
        ptA2 = g3.polar2z(dist, angle)

        self.assertTrue(np.array_equal(ptA, np.abs(ptA2)))

    def test_polygon_equivalent_radius(self):

        radius = 38.0
        expected_surface = np.pi * radius**2
        expected_perimeter = 2 * np.pi*radius

        eq_radius_perimeter = g3.regular_polygon_perimeter_equivalent_radius(4, radius)
        eq_radius_surface = g3.regular_polygon_area_equivalent_radius(4, radius)

        # eq_radius is
        perimeter = 4.0 * eq_radius_perimeter * 2**0.5

        surface = 4.0 * eq_radius_surface**2 / 2.0

        norm_rs = eq_radius_surface / radius
        norm_rp = eq_radius_perimeter / radius

        self.assertAlmostEqual(perimeter, expected_perimeter)
        self.assertAlmostEqual(surface, expected_surface)


if __name__ == '__main__':
    unittest.main()



#! /usr/bin/python
# -*- coding: utf-8 -*-

import logging

logger = logging.getLogger(__name__)

# import copy
import numpy as np
import scipy.ndimage
from scipy.sparse import csc_matrix
from . import image_manipulation as ima


class CooccurrenceMatrix(object):
    def __init__(self, data, return_counts=True, dtype=int):
        self.update_cooccurrence_matrix(data, return_counts=return_counts)
        self.dtype = dtype

    def update_cooccurrence_matrix(self, data, return_counts=True):
        self.cooccurrence_matrix = cooccurrence_matrix(data, return_counts=return_counts)

    def get(self, intensity0, intensity1):
        return self.cooccurrence_matrix[intensity0][intensity1]

    def to_ndarray(self):
        keys = self.keys()
        inv_keys = self.inv_keys()
        sz = len(self.cooccurrence_matrix)
        ndnghb = np.zeros([sz, sz], dtype=self.dtype)
        for keyx in self.cooccurrence_matrix:
            nghbx = self.cooccurrence_matrix[keyx]
            for keyy in nghbx:
                value = nghbx[keyy]
                ndnghb[inv_keys[keyx], inv_keys[keyy]] = value
        return ndnghb

    def keys(self):
        return sorted(list(self.cooccurrence_matrix.keys()))

    def inv_keys(self):
        keys = list(self.keys())
        ii = list(range(len(keys)))
        return dict(zip(keys, ii))


def cooccurrence_matrix(data, return_counts=True):
    # csc_matrix((3, 4), dtype=np.int8).toarray()

    i = 0
    nbm = {}
    it = np.nditer(data, flags=['multi_index'])
    while not it.finished:
        print("iter ", i)
        i += 1
        mindex0 = it.multi_index
        # print("%d <%s>" % (it[0], mindex0), end=' ')
        data_value0 = data[mindex0]
        for axn in range(len(mindex0)):
            mindex1 = list(mindex0)
            mindex1[axn] = mindex1[axn] + 1
            mindex1 = tuple(mindex1)
            if np.all(np.asarray(mindex1) < np.asarray(data.shape)):
                data_value1 = data[mindex1]
                # budeme vyplnovat jen spodni
                # if data_value0 < data_value1:
                #     data_value0s = data_value0
                #     data_value1s = data_value1
                # else:
                #     data_value0s = data_value1
                #     data_value1s = data_value0
                data_value0s = data_value0
                data_value1s = data_value1

                if data_value0s not in nbm.keys():
                    nbm[data_value0s] = {}
                if data_value1s not in nbm[data_value0s].keys():
                    nbm[data_value0s][data_value1s] = 0
                elif not return_counts:
                    # make it faster
                    continue

                nbm[data_value0s][data_value1s] += 1

                if data_value1s not in nbm.keys():
                    nbm[data_value1s] = {}
                if data_value0s not in nbm[data_value1s].keys():
                    nbm[data_value1s][data_value0s] = 0

                # if data_value0s != data_value1s:
                #     # we dont want to put the numer into diagonal for twice
                # on diagonal there will be doubled values
                nbm[data_value1s][data_value0s] += 1

        it.iternext()
    return nbm


def neighbors_list(labeled_ndarray, labels=None, exclude=None):
    """
    Neighbors for one or more object. Objects with label 0 are ignored.

    :param labeled_ndarray: 3D ndarray
    :param labels: Integer label or list of ints. If is set to None, all labels are processed.
    :param exclude: List of labels to exclude.
    Typically it is label of surrounding object (like liver around portal vein)
    :return:
    """

    if np.min(labeled_ndarray) < 0:
        ValueError("Input image cannot contain negative labels.")

    if exclude is None:
        exclude = []
    bboxes = scipy.ndimage.find_objects(labeled_ndarray)
    bbox_margin = 1

    if labels is None:
        labels = range(0, len(bboxes) + 1)
    else:
        if type(labels) is not list:
            labels = [labels]

    output = [None] * len(labels)

    for ilabel, label in enumerate(labels):
        # labels in bboxes starts from 1
        if label == 0:
            continue
        bbox = bboxes[label - 1]
        if bbox is not None:
            exbbox = ima.extend_crinfo(bbox, labeled_ndarray.shape, bbox_margin)
            cropped_ndarray = labeled_ndarray[exbbox]
            object = (cropped_ndarray == label)
            dilat_element = scipy.ndimage.morphology.binary_dilation(
                object,
                structure=np.ones([3, 3, 3])
            )

            neighborhood = cropped_ndarray[dilat_element]

            neighbors = np.unique(neighborhood)
            neighbors = neighbors[neighbors != label]
            # neighbors = neighbors[neighbors != 0]
            for exlabel in exclude:
                neighbors = neighbors[neighbors != exlabel]
            output[ilabel] = list(neighbors)

    return output


def get_connected_labels(neighbors_list, start_label, ignore_labels=None):
    """
    Get list of labels connected with start_label.

    :param neighbors_list: list generated by neigbors_list function (with parameter labels set to None).
    :param start_label: where is the first labeled object
    :param ignore_labels: some objects can be ignored
    :return:
    """
    if ignore_labels is None:
        ignore_labels = []

    import copy
    nl = copy.copy(neighbors_list)

    # nl.insert(0, None)
    # ignore_labels = ignore_labels
    to_process = set([start_label])
    processed = set()
    while len(to_process) > 0:
        lab = to_process.pop()
        if lab in ignore_labels:
            continue
        if lab not in processed:
            newn = nl[lab]
            to_process.update(newn)
        processed.add(lab)

    return processed

